@@ -1,6 +1,5 @@
 #!/usr/bin/env python
 from __future__ import absolute_import, print_function, unicode_literals
-
 import argparse
 import audioop
 import math
@@ -16,9 +15,8 @@ try:
     from json.decoder import JSONDecodeError
 except ImportError:
     JSONDecodeError = ValueError
-
 from progressbar import ProgressBar, Percentage, Bar, ETA
-from pygoogletranslation import Translator
+from googletrans import Translator
 import pysrt
 import six
 
@@ -27,138 +25,138 @@ GOOGLE_SPEECH_API_URL = "http://www.google.com/speech-api/v2/recognize?client=ch
 DEFAULT_USER_AGENT = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64)'
 
 arraylist_language_code = []
-arraylist_language_code.append("af");
-arraylist_language_code.append("sq");
-arraylist_language_code.append("am");
-arraylist_language_code.append("ar");
-arraylist_language_code.append("hy");
-arraylist_language_code.append("as");
-arraylist_language_code.append("ay");
-arraylist_language_code.append("az");
-arraylist_language_code.append("bm");
-arraylist_language_code.append("eu");
-arraylist_language_code.append("be");
-arraylist_language_code.append("bn");
-arraylist_language_code.append("bho");
-arraylist_language_code.append("bs");
-arraylist_language_code.append("bg");
-arraylist_language_code.append("ca");
-arraylist_language_code.append("ceb");
-arraylist_language_code.append("ny");
-arraylist_language_code.append("zh-CN");
-arraylist_language_code.append("zh-TW");
-arraylist_language_code.append("co");
-arraylist_language_code.append("cr");
-arraylist_language_code.append("cs");
-arraylist_language_code.append("da");
-arraylist_language_code.append("dv");
-arraylist_language_code.append("nl");
-arraylist_language_code.append("doi");
-arraylist_language_code.append("en");
-arraylist_language_code.append("eo");
-arraylist_language_code.append("et");
-arraylist_language_code.append("ee");
-arraylist_language_code.append("fil");
-arraylist_language_code.append("fi");
-arraylist_language_code.append("fr");
-arraylist_language_code.append("fy");
-arraylist_language_code.append("gl");
-arraylist_language_code.append("ka");
-arraylist_language_code.append("de");
-arraylist_language_code.append("el");
-arraylist_language_code.append("gn");
-arraylist_language_code.append("gu");
-arraylist_language_code.append("ht");
-arraylist_language_code.append("ha");
-arraylist_language_code.append("haw");
-arraylist_language_code.append("he");
-arraylist_language_code.append("hi");
-arraylist_language_code.append("hmn");
-arraylist_language_code.append("hu");
-arraylist_language_code.append("is");
-arraylist_language_code.append("ig");
-arraylist_language_code.append("ilo");
-arraylist_language_code.append("id");
-arraylist_language_code.append("ga");
-arraylist_language_code.append("it");
-arraylist_language_code.append("ja");
-arraylist_language_code.append("jv");
-arraylist_language_code.append("kn");
-arraylist_language_code.append("kk");
-arraylist_language_code.append("km");
-arraylist_language_code.append("rw");
-arraylist_language_code.append("kok");
-arraylist_language_code.append("ko");
-arraylist_language_code.append("kri");
-arraylist_language_code.append("kmr");
-arraylist_language_code.append("ckb");
-arraylist_language_code.append("ky");
-arraylist_language_code.append("lo");
-arraylist_language_code.append("la");
-arraylist_language_code.append("lv");
-arraylist_language_code.append("ln");
-arraylist_language_code.append("lt");
-arraylist_language_code.append("lg");
-arraylist_language_code.append("lb");
-arraylist_language_code.append("mk");
-arraylist_language_code.append("mg");
-arraylist_language_code.append("ms");
-arraylist_language_code.append("ml");
-arraylist_language_code.append("mt");
-arraylist_language_code.append("mi");
-arraylist_language_code.append("mr");
-arraylist_language_code.append("mni");
-arraylist_language_code.append("lus");
-arraylist_language_code.append("mn");
-arraylist_language_code.append("mmr");
-arraylist_language_code.append("ne");
-arraylist_language_code.append("no");
-arraylist_language_code.append("or");
-arraylist_language_code.append("om");
-arraylist_language_code.append("ps");
-arraylist_language_code.append("fa");
-arraylist_language_code.append("pl");
-arraylist_language_code.append("pt");
-arraylist_language_code.append("pa");
-arraylist_language_code.append("qu");
-arraylist_language_code.append("ro");
-arraylist_language_code.append("ru");
-arraylist_language_code.append("sm");
-arraylist_language_code.append("sa");
-arraylist_language_code.append("gd");
-arraylist_language_code.append("nso");
-arraylist_language_code.append("sr");
-arraylist_language_code.append("st");
-arraylist_language_code.append("sn");
-arraylist_language_code.append("sd");
-arraylist_language_code.append("si");
-arraylist_language_code.append("sk");
-arraylist_language_code.append("sl");
-arraylist_language_code.append("so");
-arraylist_language_code.append("es");
-arraylist_language_code.append("su");
-arraylist_language_code.append("sw");
-arraylist_language_code.append("sv");
-arraylist_language_code.append("tg");
-arraylist_language_code.append("ta");
-arraylist_language_code.append("tt");
-arraylist_language_code.append("te");
-arraylist_language_code.append("th");
-arraylist_language_code.append("ti");
-arraylist_language_code.append("ts");
-arraylist_language_code.append("tr");
-arraylist_language_code.append("tk");
-arraylist_language_code.append("tw");
-arraylist_language_code.append("ug");
-arraylist_language_code.append("uk");
-arraylist_language_code.append("ur");
-arraylist_language_code.append("uz");
-arraylist_language_code.append("vi");
-arraylist_language_code.append("cy");
-arraylist_language_code.append("xh");
-arraylist_language_code.append("yi");
-arraylist_language_code.append("yo");
-arraylist_language_code.append("zu");
+arraylist_language_code.append("af")
+arraylist_language_code.append("sq")
+arraylist_language_code.append("am")
+arraylist_language_code.append("ar")
+arraylist_language_code.append("hy")
+arraylist_language_code.append("as")
+arraylist_language_code.append("ay")
+arraylist_language_code.append("az")
+arraylist_language_code.append("bm")
+arraylist_language_code.append("eu")
+arraylist_language_code.append("be")
+arraylist_language_code.append("bn")
+arraylist_language_code.append("bho")
+arraylist_language_code.append("bs")
+arraylist_language_code.append("bg")
+arraylist_language_code.append("ca")
+arraylist_language_code.append("ceb")
+arraylist_language_code.append("ny")
+arraylist_language_code.append("zh-CN")
+arraylist_language_code.append("zh-TW")
+arraylist_language_code.append("co")
+arraylist_language_code.append("hr")
+arraylist_language_code.append("cs")
+arraylist_language_code.append("da")
+arraylist_language_code.append("dv")
+arraylist_language_code.append("doi")
+arraylist_language_code.append("nl")
+arraylist_language_code.append("en")
+arraylist_language_code.append("eo")
+arraylist_language_code.append("et")
+arraylist_language_code.append("ee")
+arraylist_language_code.append("fil")
+arraylist_language_code.append("fi")
+arraylist_language_code.append("fr")
+arraylist_language_code.append("fy")
+arraylist_language_code.append("gl")
+arraylist_language_code.append("ka")
+arraylist_language_code.append("de")
+arraylist_language_code.append("el")
+arraylist_language_code.append("gn")
+arraylist_language_code.append("gu")
+arraylist_language_code.append("ht")
+arraylist_language_code.append("ha")
+arraylist_language_code.append("haw")
+arraylist_language_code.append("he")
+arraylist_language_code.append("hi")
+arraylist_language_code.append("hmn")
+arraylist_language_code.append("hu")
+arraylist_language_code.append("is")
+arraylist_language_code.append("ig")
+arraylist_language_code.append("ilo")
+arraylist_language_code.append("id")
+arraylist_language_code.append("ga")
+arraylist_language_code.append("it")
+arraylist_language_code.append("ja")
+arraylist_language_code.append("jv")
+arraylist_language_code.append("kn")
+arraylist_language_code.append("kk")
+arraylist_language_code.append("km")
+arraylist_language_code.append("rw")
+arraylist_language_code.append("gom")
+arraylist_language_code.append("ko")
+arraylist_language_code.append("kri")
+arraylist_language_code.append("kmr")
+arraylist_language_code.append("ckb")
+arraylist_language_code.append("ky")
+arraylist_language_code.append("lo")
+arraylist_language_code.append("la")
+arraylist_language_code.append("lv")
+arraylist_language_code.append("ln")
+arraylist_language_code.append("lt")
+arraylist_language_code.append("lg")
+arraylist_language_code.append("lb")
+arraylist_language_code.append("mk")
+arraylist_language_code.append("mg")
+arraylist_language_code.append("ms")
+arraylist_language_code.append("ml")
+arraylist_language_code.append("mt")
+arraylist_language_code.append("mi")
+arraylist_language_code.append("mr")
+arraylist_language_code.append("mni-Mtei")
+arraylist_language_code.append("lus")
+arraylist_language_code.append("mn")
+arraylist_language_code.append("my")
+arraylist_language_code.append("ne")
+arraylist_language_code.append("no")
+arraylist_language_code.append("or")
+arraylist_language_code.append("om")
+arraylist_language_code.append("ps")
+arraylist_language_code.append("fa")
+arraylist_language_code.append("pl")
+arraylist_language_code.append("pt")
+arraylist_language_code.append("pa")
+arraylist_language_code.append("qu")
+arraylist_language_code.append("ro")
+arraylist_language_code.append("ru")
+arraylist_language_code.append("sm")
+arraylist_language_code.append("sa")
+arraylist_language_code.append("gd")
+arraylist_language_code.append("nso")
+arraylist_language_code.append("sr")
+arraylist_language_code.append("st")
+arraylist_language_code.append("sn")
+arraylist_language_code.append("sd")
+arraylist_language_code.append("si")
+arraylist_language_code.append("sk")
+arraylist_language_code.append("sl")
+arraylist_language_code.append("so")
+arraylist_language_code.append("es")
+arraylist_language_code.append("su")
+arraylist_language_code.append("sw")
+arraylist_language_code.append("sv")
+arraylist_language_code.append("tg")
+arraylist_language_code.append("ta")
+arraylist_language_code.append("tt")
+arraylist_language_code.append("te")
+arraylist_language_code.append("th")
+arraylist_language_code.append("ti")
+arraylist_language_code.append("ts")
+arraylist_language_code.append("tr")
+arraylist_language_code.append("tk")
+arraylist_language_code.append("tw")
+arraylist_language_code.append("uk")
+arraylist_language_code.append("ur")
+arraylist_language_code.append("ug")
+arraylist_language_code.append("uz")
+arraylist_language_code.append("vi")
+arraylist_language_code.append("cy")
+arraylist_language_code.append("xh")
+arraylist_language_code.append("yi")
+arraylist_language_code.append("yo")
+arraylist_language_code.append("zu")
 
 arraylist_language = []
 arraylist_language.append("Afrikaans");
@@ -172,20 +170,20 @@ arraylist_language.append("Azerbaijani");
 arraylist_language.append("Bambara");
 arraylist_language.append("Basque");
 arraylist_language.append("Belarusian");
-arraylist_language.append("Bengali (Bangla)");
+arraylist_language.append("Bengali");
 arraylist_language.append("Bhojpuri");
 arraylist_language.append("Bosnian");
 arraylist_language.append("Bulgarian");
 arraylist_language.append("Catalan");
 arraylist_language.append("Cebuano");
-arraylist_language.append("Chichewa, Nyanja");
+arraylist_language.append("Chichewa");
 arraylist_language.append("Chinese (Simplified)");
 arraylist_language.append("Chinese (Traditional)");
 arraylist_language.append("Corsican");
 arraylist_language.append("Croatian");
 arraylist_language.append("Czech");
 arraylist_language.append("Danish");
-arraylist_language.append("Divehi, Maldivian");
+arraylist_language.append("Dhivehi");
 arraylist_language.append("Dogri");
 arraylist_language.append("Dutch");
 arraylist_language.append("English");
@@ -220,7 +218,7 @@ arraylist_language.append("Javanese");
 arraylist_language.append("Kannada");
 arraylist_language.append("Kazakh");
 arraylist_language.append("Khmer");
-arraylist_language.append("Kinyarwanda (Rwanda)");
+arraylist_language.append("Kinyarwanda");
 arraylist_language.append("Konkani");
 arraylist_language.append("Korean");
 arraylist_language.append("Krio");
@@ -229,10 +227,10 @@ arraylist_language.append("Kurdish (Sorani)");
 arraylist_language.append("Kyrgyz");
 arraylist_language.append("Lao");
 arraylist_language.append("Latin");
-arraylist_language.append("Latvian (Lettish)");
+arraylist_language.append("Latvian");
 arraylist_language.append("Lingala");
 arraylist_language.append("Lithuanian");
-arraylist_language.append("Luganda, Ganda");
+arraylist_language.append("Luganda");
 arraylist_language.append("Luxembourgish");
 arraylist_language.append("Macedonian");
 arraylist_language.append("Malagasy");
@@ -247,15 +245,15 @@ arraylist_language.append("Mongolian");
 arraylist_language.append("Myanmar (Burmese)");
 arraylist_language.append("Nepali");
 arraylist_language.append("Norwegian");
-arraylist_language.append("Oriya");
-arraylist_language.append("Oromo (Afaan Oromo)");
-arraylist_language.append("Pashto, Pushto");
-arraylist_language.append("Persian (Farsi)");
+arraylist_language.append("Odiya (Oriya)");
+arraylist_language.append("Oromo");
+arraylist_language.append("Pashto");
+arraylist_language.append("Persian");
 arraylist_language.append("Polish");
 arraylist_language.append("Portuguese");
-arraylist_language.append("Punjabi (Eastern)");
+arraylist_language.append("Punjabi");
 arraylist_language.append("Quechua");
-arraylist_language.append("Romanian, Moldavian");
+arraylist_language.append("Romanian");
 arraylist_language.append("Russian");
 arraylist_language.append("Samoan");
 arraylist_language.append("Sanskrit");
@@ -265,13 +263,13 @@ arraylist_language.append("Serbian");
 arraylist_language.append("Sesotho");
 arraylist_language.append("Shona");
 arraylist_language.append("Sindhi");
-arraylist_language.append("Sinhalese");
+arraylist_language.append("Sinhala");
 arraylist_language.append("Slovak");
 arraylist_language.append("Slovenian");
 arraylist_language.append("Somali");
 arraylist_language.append("Spanish");
 arraylist_language.append("Sundanese");
-arraylist_language.append("Swahili (Kiswahili)");
+arraylist_language.append("Swahili");
 arraylist_language.append("Swedish");
 arraylist_language.append("Tajik");
 arraylist_language.append("Tamil");
@@ -282,7 +280,7 @@ arraylist_language.append("Tigrinya");
 arraylist_language.append("Tsonga");
 arraylist_language.append("Turkish");
 arraylist_language.append("Turkmen");
-arraylist_language.append("Twi");
+arraylist_language.append("Twi (Akan)");
 arraylist_language.append("Ukrainian");
 arraylist_language.append("Urdu");
 arraylist_language.append("Uyghur");
@@ -426,7 +424,7 @@ class SpeechRecognizer(object):
             return
 
 
-class TranscriptTranslator(object):
+class SentenceTranslator(object):
     def __init__(self, src, dest, patience=-1):
         self.src = src
         self.dest = dest
@@ -702,7 +700,7 @@ def main():
     #parser.add_argument('-n', '--rename', type=str, help='rename the output file.')
     #parser.add_argument('-p', '--patience', type=int, help='the patience of retrying to translate. Expect a positive number.  If -1 is assigned, the program will try for infinite times until there is no failures happened in the output.')
     #parser.add_argument('-V', '--verbose', action="store_true", help='logs the translation process to console.')
-    parser.add_argument('-v', '--version', action='version', version='1.0.1')
+    parser.add_argument('-v', '--version', action='version', version='1.0.6')
     parser.add_argument('-lf', '--list-formats', help="List all available subtitle formats", action='store_true')
     parser.add_argument('-ll', '--list-languages', help="List all available source/destination languages", action='store_true')
 
@@ -875,12 +873,13 @@ def main():
                     f.write('\n')
         '''
 
+        '''
         # ARGUMENT VERBOSE REMOVED
-        # CONCURRENT TRANSLATION USING class TranscriptTranslator
+        # CONCURRENT TRANSLATION USING class SentenceTranslator
         prompt = "Translating from %5s to %5s         : " %(args.src_language, args.dst_language)
         widgets = [prompt, Percentage(), ' ', Bar(), ' ', ETA()]
         pbar = ProgressBar(widgets=widgets, maxval=len(transcripts)).start()
-        sentence_translator = TranscriptTranslator(src=args.src_language, dest=args.dst_language)
+        sentence_translator = SentenceTranslator(src=args.src_language, dest=args.dst_language)
         translated_transcripts = []
         for i, translated_transcript in enumerate(pool.imap(sentence_translator, transcripts)):
             translated_transcripts.append(translated_transcript)
@@ -895,6 +894,37 @@ def main():
             f.write(formatted_translated_subtitles.encode("utf-8"))
         with open(translated_subtitle_file, 'a') as f:
             f.write("\n")
+        '''
+
+        # ARGUMENT VERBOSE REMOVED
+        # AND CONCURRENT TRANSLATION USING class SentenceTranslator(object)
+        # BUT NO NEED TO TRANSLATE ALL transcript IN transcripts (BECAUSE SOME region in regions MAY JUST HAVE EMPTY STRING transcript)
+        # JUST TRANSLATE ALREADY CREATED subtitles ENTRIES FROM timed_subtitles
+        created_regions = []
+        created_subtitles = []
+        for entry in timed_subtitles:
+            created_regions.append(entry[0])
+            created_subtitles.append(entry[1])
+
+        prompt = "Translating from %5s to %5s         : " %(args.src_language, args.dst_language)
+        widgets = [prompt, Percentage(), ' ', Bar(), ' ', ETA()]
+        pbar = ProgressBar(widgets=widgets, maxval=len(timed_subtitles)).start()
+        transcript_translator = SentenceTranslator(src=args.src_language, dest=args.dst_language)
+        translated_subtitles = []
+        for i, translated_subtitle in enumerate(pool.imap(transcript_translator, created_subtitles)):
+            translated_subtitles.append(translated_subtitle)
+            pbar.update(i)
+        pbar.finish()
+
+        timed_translated_subtitles = [(r, t) for r, t in zip(created_regions, translated_subtitles) if t]
+        formatter = FORMATTERS.get(args.format)
+        formatted_translated_subtitles = formatter(timed_translated_subtitles)
+
+        with open(translated_subtitle_file, 'wb') as tf:
+            tf.write(formatted_translated_subtitles.encode("utf-8"))
+
+        with open(translated_subtitle_file, 'a') as tf:
+            tf.write("\n")
 
     os.remove(audio_filename)
 
